@@ -28,6 +28,9 @@
 ****************************************************************************/
 
 #include "lupdate.h"
+#if QT_CONFIG(clangcpp)
+#include "cpp_clang.h"
+#endif
 
 #include <profileutils.h>
 #include <projectdescriptionreader.h>
@@ -37,7 +40,6 @@
 
 #include <QtCore/QCoreApplication>
 #include <QtCore/QDir>
-#include <QtCore/QDirIterator>
 #include <QtCore/QFile>
 #include <QtCore/QFileInfo>
 #include <QtCore/QLibraryInfo>
@@ -47,6 +49,10 @@
 
 #include <iostream>
 
+bool useClangToParseCpp = false;
+QString commandLineCompilationDatabaseDir; // for the path to the json file passed as a command line argument.
+                                    // Has priority over what is in the .pro file and passed to the project.
+
 // Can't have an array of QStaticStringData<N> for different N, so
 // use QString, which requires constructor calls. Doesn't matter
 // much, since this is in an app, not a lib:
@@ -84,9 +90,8 @@ int TrFunctionAliasManager::trFunctionByName(const QString &trFunctionName) cons
 {
     ensureTrFunctionHashUpdated();
     // this function needs to be fast
-    const QHash<QString, TrFunction>::const_iterator it
-        = m_nameToTrFunctionMap.find(trFunctionName);
-    return it == m_nameToTrFunctionMap.end() ? -1 : *it;
+    const auto it = m_nameToTrFunctionMap.constFind(trFunctionName);
+    return it == m_nameToTrFunctionMap.cend() ? -1 : *it;
 }
 
 void TrFunctionAliasManager::modifyAlias(int trFunction, const QString &alias, Operation op)
@@ -105,7 +110,7 @@ void TrFunctionAliasManager::ensureTrFunctionHashUpdated() const
 
     QHash<QString, TrFunction> nameToTrFunctionMap;
     for (int i = 0; i < NumTrFunctions; ++i)
-        foreach (const QString &alias, m_trFunctionAliases[i])
+        for (const QString &alias : m_trFunctionAliases[i])
             nameToTrFunctionMap[alias] = TrFunction(i);
     // commit:
     m_nameToTrFunctionMap.swap(nameToTrFunctionMap);
@@ -197,15 +202,11 @@ static void printErr(const QString & out)
     std::cerr << qPrintable(out);
 }
 
-class LU {
-    Q_DECLARE_TR_FUNCTIONS(LUpdate)
-};
-
 static void recursiveFileInfoList(const QDir &dir,
     const QSet<QString> &nameFilters, QDir::Filters filter,
     QFileInfoList *fileinfolist)
 {
-    foreach (const QFileInfo &fi, dir.entryInfoList(filter))
+    for (const QFileInfo &fi : dir.entryInfoList(filter))
         if (fi.isDir())
             recursiveFileInfoList(QDir(fi.absoluteFilePath()), nameFilters, filter, fileinfolist);
         else if (nameFilters.contains(fi.suffix()))
@@ -279,6 +280,16 @@ static void printUsage()
         "           Specify the output file(s). This will override the TRANSLATIONS.\n"
         "    -version\n"
         "           Display the version of lupdate and exit.\n"
+        "    -clang-parser [compilation-database-dir]\n"
+        "           Use clang to parse cpp files. Otherwise a custom parser is used.\n"
+        "           This option needs a clang compilation database (compile_commands.json)\n"
+        "           for the files that needs to be parsed.\n"
+        "           The path to the directory containing this file can be specified on the \n"
+        "           command line, directly after the -clang-parser option, or in the .pro file\n"
+        "           by setting the variable LUPDATE_COMPILE_COMMANDS_PATH.\n"
+        "           A directory specified on the command line takes precedence.\n"
+        "           If no path is given, the compilation database will be searched\n"
+        "           in all parent paths of the first input file.\n"
         "    @lst-file\n"
         "           Read additional file names (one per line) or includepaths (one per\n"
         "           line, and prefixed with -I) from lst-file.\n"
@@ -289,7 +300,7 @@ static void printUsage()
 
 static bool handleTrFunctionAliases(const QString &arg)
 {
-    foreach (const QString &pair, arg.split(QLatin1Char(','), QString::SkipEmptyParts)) {
+    for (const QString &pair : arg.split(QLatin1Char(','), Qt::SkipEmptyParts)) {
         const int equalSign = pair.indexOf(QLatin1Char('='));
         if (equalSign < 0) {
             printErr(LU::tr("tr-function mapping '%1' in -tr-function-alias is missing the '='.\n").arg(pair));
@@ -330,7 +341,7 @@ static void updateTsFiles(const Translator &fetchedTor, const QStringList &tsFil
     }
 
     QList<Translator> aliens;
-    foreach (const QString &fileName, alienFiles) {
+    for (const QString &fileName : alienFiles) {
         ConversionData cd;
         Translator tor;
         if (!tor.load(fileName, cd, QLatin1String("auto"))) {
@@ -344,7 +355,7 @@ static void updateTsFiles(const Translator &fetchedTor, const QStringList &tsFil
 
     QDir dir;
     QString err;
-    foreach (const QString &fileName, tsFileNames) {
+    for (const QString &fileName : tsFileNames) {
         QString fn = dir.relativeFilePath(fileName);
         ConversionData cd;
         Translator tor;
@@ -466,11 +477,11 @@ static QStringList getResources(const QString &resourceFile)
 
 static bool processTs(Translator &fetchedTor, const QString &file, ConversionData &cd)
 {
-    foreach (const Translator::FileFormat &fmt, Translator::registeredFileFormats()) {
+    for (const Translator::FileFormat &fmt : qAsConst(Translator::registeredFileFormats())) {
         if (file.endsWith(QLatin1Char('.') + fmt.extension, Qt::CaseInsensitive)) {
             Translator tor;
             if (tor.load(file, cd, fmt.extension)) {
-                foreach (TranslatorMessage msg, tor.messages()) {
+                for (TranslatorMessage msg : tor.messages()) {
                     msg.setType(TranslatorMessage::Unfinished);
                     msg.setTranslations(QStringList());
                     msg.setTranslatorComment(QString());
@@ -484,32 +495,32 @@ static bool processTs(Translator &fetchedTor, const QString &file, ConversionDat
 }
 
 static void processSources(Translator &fetchedTor,
-                           const QStringList &sourceFiles, ConversionData &cd)
+                           const QStringList &sourceFiles, ConversionData &cd, bool *fail)
 {
 #ifdef QT_NO_QML
     bool requireQmlSupport = false;
 #endif
     QStringList sourceFilesCpp;
-    for (QStringList::const_iterator it = sourceFiles.begin(); it != sourceFiles.end(); ++it) {
-        if (it->endsWith(QLatin1String(".java"), Qt::CaseInsensitive))
-            loadJava(fetchedTor, *it, cd);
-        else if (it->endsWith(QLatin1String(".ui"), Qt::CaseInsensitive)
-                 || it->endsWith(QLatin1String(".jui"), Qt::CaseInsensitive))
-            loadUI(fetchedTor, *it, cd);
+    for (const auto &sourceFile : sourceFiles) {
+        if (sourceFile.endsWith(QLatin1String(".java"), Qt::CaseInsensitive))
+            loadJava(fetchedTor, sourceFile, cd);
+        else if (sourceFile.endsWith(QLatin1String(".ui"), Qt::CaseInsensitive)
+                 || sourceFile.endsWith(QLatin1String(".jui"), Qt::CaseInsensitive))
+            loadUI(fetchedTor, sourceFile, cd);
 #ifndef QT_NO_QML
-        else if (it->endsWith(QLatin1String(".js"), Qt::CaseInsensitive)
-                 || it->endsWith(QLatin1String(".qs"), Qt::CaseInsensitive))
-            loadQScript(fetchedTor, *it, cd);
-        else if (it->endsWith(QLatin1String(".qml"), Qt::CaseInsensitive))
-            loadQml(fetchedTor, *it, cd);
+        else if (sourceFile.endsWith(QLatin1String(".js"), Qt::CaseInsensitive)
+                 || sourceFile.endsWith(QLatin1String(".qs"), Qt::CaseInsensitive))
+            loadQScript(fetchedTor, sourceFile, cd);
+        else if (sourceFile.endsWith(QLatin1String(".qml"), Qt::CaseInsensitive))
+            loadQml(fetchedTor, sourceFile, cd);
 #else
-        else if (it->endsWith(QLatin1String(".qml"), Qt::CaseInsensitive)
-                 || it->endsWith(QLatin1String(".js"), Qt::CaseInsensitive)
-                 || it->endsWith(QLatin1String(".qs"), Qt::CaseInsensitive))
+        else if (sourceFile.endsWith(QLatin1String(".qml"), Qt::CaseInsensitive)
+                 || sourceFile.endsWith(QLatin1String(".js"), Qt::CaseInsensitive)
+                 || sourceFile.endsWith(QLatin1String(".qs"), Qt::CaseInsensitive))
             requireQmlSupport = true;
 #endif // QT_NO_QML
-        else if (!processTs(fetchedTor, *it, cd))
-            sourceFilesCpp << *it;
+        else if (!processTs(fetchedTor, sourceFile, cd))
+            sourceFilesCpp << sourceFile;
     }
 
 #ifdef QT_NO_QML
@@ -517,7 +528,17 @@ static void processSources(Translator &fetchedTor,
         printErr(LU::tr("lupdate warning: Some files have been ignored due to missing qml/javascript support\n"));
 #endif
 
-    loadCPP(fetchedTor, sourceFilesCpp, cd);
+    if (useClangToParseCpp) {
+#if QT_CONFIG(clangcpp)
+        ClangCppParser::loadCPP(fetchedTor, sourceFilesCpp, cd, fail);
+#else
+        *fail = true;
+        printErr(LU::tr("lupdate error: lupdate was built without clang support."));
+#endif
+    }
+    else
+        loadCPP(fetchedTor, sourceFilesCpp, cd);
+
     if (!cd.error().isEmpty())
         printErr(cd.error());
 }
@@ -529,14 +550,14 @@ static QSet<QString> projectRoots(const QString &projectFile, const QStringList
     sourceDirs.insert(proPath + QLatin1Char('/'));
     for (const QString &sf : sourceFiles)
         sourceDirs.insert(sf.left(sf.lastIndexOf(QLatin1Char('/')) + 1));
-    QStringList rootList = sourceDirs.toList();
+    QStringList rootList = sourceDirs.values();
     rootList.sort();
     for (int prev = 0, curr = 1; curr < rootList.length(); )
         if (rootList.at(curr).startsWith(rootList.at(prev)))
             rootList.removeAt(curr);
         else
             prev = curr++;
-    return rootList.toSet();
+    return QSet<QString>(rootList.cbegin(), rootList.cend());
 }
 
 class ProjectProcessor
@@ -583,9 +604,13 @@ private:
         cd.m_includePath = prj.includePaths;
         cd.m_excludes = prj.excluded;
         cd.m_sourceIsUtf16 = options & SourceIsUtf16;
+        if (commandLineCompilationDatabaseDir.isEmpty())
+            cd.m_compilationDatabaseDir = prj.compileCommands;
+        else
+            cd.m_compilationDatabaseDir = commandLineCompilationDatabaseDir;
 
         QStringList tsFiles;
-        if (hasTranslations(prj)) {
+        if (prj.translations) {
             tsFiles = *prj.translations;
             if (parentTor) {
                 if (topLevel) {
@@ -606,7 +631,7 @@ private:
             }
             Translator tor;
             processProjects(false, options, prj.subProjects, false, &tor, fail);
-            processSources(tor, sources, cd);
+            processSources(tor, sources, cd, fail);
             updateTsFiles(tor, tsFiles, QStringList(), m_sourceLanguage, m_targetLanguage,
                           options, fail);
             return;
@@ -620,10 +645,10 @@ private:
             }
             Translator tor;
             processProjects(false, options, prj.subProjects, nestComplain, &tor, fail);
-            processSources(tor, sources, cd);
+            processSources(tor, sources, cd, fail);
         } else {
             processProjects(false, options, prj.subProjects, nestComplain, parentTor, fail);
-            processSources(*parentTor, sources, cd);
+            processSources(*parentTor, sources, cd, fail);
         }
     }
 
@@ -639,7 +664,7 @@ int main(int argc, char **argv)
     QTranslator translator;
     QTranslator qtTranslator;
     QString sysLocale = QLocale::system().name();
-    QString resourceDir = QLibraryInfo::location(QLibraryInfo::TranslationsPath);
+    QString resourceDir = QLibraryInfo::path(QLibraryInfo::TranslationsPath);
     if (translator.load(QLatin1String("linguist_") + sysLocale, resourceDir)
         && qtTranslator.load(QLatin1String("qt_") + sysLocale, resourceDir)) {
         app.installTranslator(&translator);
@@ -837,7 +862,19 @@ int main(int argc, char **argv)
                 includePath += args[i].mid(2);
             }
             continue;
-        } else if (arg.startsWith(QLatin1String("-")) && arg != QLatin1String("-")) {
+        }
+#if QT_CONFIG(clangcpp)
+        else if (arg == QLatin1String("-clang-parser")) {
+            useClangToParseCpp = true;
+            // the option after -clang-parser is optional
+            if ((i + 1) != argc && !args[i + 1].startsWith(QLatin1String("-"))) {
+                 i++;
+                 commandLineCompilationDatabaseDir = args[i];
+             }
+            continue;
+        }
+#endif
+        else if (arg.startsWith(QLatin1String("-")) && arg != QLatin1String("-")) {
             printErr(LU::tr("Unrecognized option '%1'.\n").arg(arg));
             return 1;
         }
@@ -853,7 +890,7 @@ int main(int argc, char **argv)
             while (!lstFile.atEnd()) {
                 QString lineContent = QString::fromLocal8Bit(lstFile.readLine().trimmed());
 
-                if (lineContent.startsWith(QLatin1Literal("-I"))) {
+                if (lineContent.startsWith(QLatin1String("-I"))) {
                     if (lineContent.length() == 2) {
                         printErr(LU::tr("The -I option should be followed by a path.\n"));
                         return 1;
@@ -867,9 +904,9 @@ int main(int argc, char **argv)
             files << arg;
         }
         if (metTsFlag) {
-            foreach (const QString &file, files) {
+            for (const QString &file : qAsConst(files)) {
                 bool found = false;
-                foreach (const Translator::FileFormat &fmt, Translator::registeredFileFormats()) {
+                for (const Translator::FileFormat &fmt : qAsConst(Translator::registeredFileFormats())) {
                     if (file.endsWith(QLatin1Char('.') + fmt.extension, Qt::CaseInsensitive)) {
                         QFileInfo fi(file);
                         if (!fi.exists() || fi.isWritable()) {
@@ -892,7 +929,7 @@ int main(int argc, char **argv)
         } else if (metXTsFlag) {
             alienFiles += files;
         } else {
-            foreach (const QString &file, files) {
+            for (const QString &file : qAsConst(files)) {
                 QFileInfo fi(file);
                 if (!fi.exists()) {
                     printErr(LU::tr("lupdate error: File '%1' does not exist.\n").arg(file));
@@ -907,7 +944,7 @@ int main(int argc, char **argv)
                     QDir dir = QDir(fi.filePath());
                     projectRoots.insert(dir.absolutePath() + QLatin1Char('/'));
                     if (extensionsNameFilters.isEmpty()) {
-                        foreach (QString ext, extensions.split(QLatin1Char(','))) {
+                        for (QString ext : extensions.split(QLatin1Char(','))) {
                             ext = ext.trimmed();
                             if (ext.startsWith(QLatin1Char('.')))
                                 ext.remove(0, 1);
@@ -920,7 +957,7 @@ int main(int argc, char **argv)
                     QFileInfoList fileinfolist;
                     recursiveFileInfoList(dir, extensionsNameFilters, filters, &fileinfolist);
                     int scanRootLen = dir.absolutePath().length();
-                    foreach (const QFileInfo &fi, fileinfolist) {
+                    for (const QFileInfo &fi : qAsConst(fileinfolist)) {
                         QString fn = QDir::cleanPath(fi.absoluteFilePath());
                         if (fn.endsWith(QLatin1String(".qrc"), Qt::CaseInsensitive)) {
                             resourceFiles << fn;
@@ -967,7 +1004,7 @@ int main(int argc, char **argv)
 
     QString errorString;
     if (!proFiles.isEmpty()) {
-        runQtTool(QStringLiteral("lupdate-pro"), app.arguments().mid(1));
+        runInternalQtTool(QStringLiteral("lupdate-pro"), app.arguments().mid(1));
         return 0;
     }
 
@@ -999,9 +1036,10 @@ int main(int argc, char **argv)
         cd.m_projectRoots = projectRoots;
         cd.m_includePath = includePath;
         cd.m_allCSources = allCSources;
+        cd.m_compilationDatabaseDir = commandLineCompilationDatabaseDir;
         for (const QString &resource : qAsConst(resourceFiles))
             sourceFiles << getResources(resource);
-        processSources(fetchedTor, sourceFiles, cd);
+        processSources(fetchedTor, sourceFiles, cd, &fail);
         updateTsFiles(fetchedTor, tsFileNames, alienFiles,
                       sourceLanguage, targetLanguage, options, &fail);
     } else {
