@@ -285,9 +285,7 @@ void Section::reduce()
 {
     if (!isEmpty()) {
         keys_ = memberMap_.keys();
-        obsoleteKeys_ = obsoleteMemberMap_.keys();
         members_ = memberMap_.values().toVector();
-        obsoleteMembers_ = obsoleteMemberMap_.values().toVector();
         reimplementedMembers_ = reimplementedMemberMap_.values().toVector();
         for (int i = 0; i < classMapList_.size(); i++) {
             ClassMap *cm = classMapList_[i];
@@ -298,6 +296,10 @@ void Section::reduce()
             classKeysNodesList_.append(ckn);
         }
     }
+    if (!obsoleteMemberMap_.isEmpty()) {
+        obsoleteKeys_ = obsoleteMemberMap_.keys();
+        obsoleteMembers_ = obsoleteMemberMap_.values().toVector();
+    }
 }
 
 /*!
@@ -1012,6 +1014,21 @@ void Sections::buildStdQmlTypeRefPageSections()
             if (n->isInternal())
                 continue;
 
+            // Skip overridden property/function documentation from abstract base type
+            if (qtn != aggregate_ && qtn->isAbstract()) {
+                NodeList candidates;
+                aggregate_->findChildren(n->name(), candidates);
+                if (std::any_of(candidates.cbegin(), candidates.cend(), [&n](const Node *c) {
+                    if (c->nodeType() == n->nodeType()) {
+                        if (!n->isFunction() || static_cast<const FunctionNode*>(n)->compare(c, false))
+                            return true;
+                    }
+                    return false;
+                })) {
+                    continue;
+                }
+            }
+
             if (!n->isSharedCommentNode() || n->isPropertyGroup())
                 allMembers.add(classMap, n);
 
